﻿using System;
using System.Linq;

using IdeaBlade.EntityModel;
using IdeaBlade.Core;

namespace DomainModel {
  /// <summary>
  /// The developer class representing the Customer business object.
  /// </summary>
  public partial class Customer {

     #region PropertyInterceptors

    // Force the City to upper case for display purposes.
    [AfterGet(EntityPropertyNames.City)]
    public String UppercaseCity(String city) {
      if (!String.IsNullOrEmpty(city)) {
        return city.ToUpper();
      } else {
        return city;
      }
    }

    // Log the set of any customer property.
    [AfterSet]
    public void AfterSetAnyProperty(IEntityPropertyInterceptorArgs args) {
      string msg = string.Format("Setting {0} to {1}", args.EntityProperty.Name,
        args.Value == null ? "null" : args.Value.ToString());
      TraceLogger(msg);
    }

    // Force the ContactName to upper case when set.
    [BeforeSet("ContactName", Key = "C1")]
    public void BeforeSetContactName(DataEntityPropertySetInterceptorArgs<Customer, string> args) {
      if (args.Value != null) {
        args.Value = args.Value.ToUpper();
      }
    }

    // Log when ContactName is set.
    [AfterSet("ContactName", Key = "C2")]
    public void AfterSetContactName(DataEntityPropertySetInterceptorArgs<Customer, string> args) {
      string msg = string.Format("Set {0} to {1}", args.DataEntityProperty.Name, 
        args.Value == null ? "null" : args.Value);
      TraceLogger(msg);
    }


    private static void TraceLogger(string msg) {
      TraceFns.WriteLine(msg);
    }

    #endregion

  }

}
