﻿using System;
using System.Linq;

using IdeaBlade.EntityModel;
using IdeaBlade.Core;

namespace DomainModel {
  /// <summary>
  /// The developer class representing the Employee business object.
  /// </summary>
  public partial class Employee  {
     
  
    // Add additional logic to your business object here...
      #region Public Properties

      /// <summary>
      /// LastName, FirstName
      /// </summary>
      public String LastFirst {
          get {
              String aLastFirst = this.LastName + ", " + this.FirstName;
              if (aLastFirst == ", ") aLastFirst = "(Not specified)";
              return aLastFirst;
          }
      }

      /// <summary>
      /// Age as of today
      /// </summary>
      public int Age {
          get {
              if (null == BirthDate) return 0;
              DateTime oBirthDate = (DateTime)this.BirthDate;
              DateTime oToday = DateTime.Today;
              int oAge = oToday.Year - oBirthDate.Year;
              if (oBirthDate.AddYears(oAge) > oToday) oAge--;
              if (oAge < 0) return 0;
              else return oAge;
          }
      }

      public int NumberOfOrders {
          get {
              return this.Orders.Count();
          }
      }

      /// <summary>
      /// Total revenue for this Employee's orders
      /// </summary>
      public double TotalOrderRevenue {
          get {
              double revenue = 0;
              foreach (Order aOrder in this.Orders) {
                  foreach (OrderDetail aOrderDetail in aOrder.OrderDetails) {
                      revenue += aOrderDetail.Quantity * Convert.ToDouble(aOrderDetail.UnitPrice) * aOrderDetail.Discount;
                  }
              }
              return revenue;
          }
      }

      #endregion
      
  }
  
}
