﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Windows.Controls;

using DomainModel;
using IdeaBlade.EntityModel;
using System.Collections.ObjectModel;

namespace LiteHausWPF {
    /// <summary>
    /// Interaction logic for EmployeesUserControl.xaml
    /// </summary>
    public partial class EmployeesUserControl {
        public EmployeesUserControl() {
            this.InitializeComponent();
            ConfigureEmployeeSelectorListBox();
            ConfigureManagerComboBox();
        }

        private void ConfigureEmployeeSelectorListBox() {
            //_selectEmployeeListBox.DisplayMemberPath = Employee.PathFor(e => e.LastFirst);
            _selectEmployeeListBox.SelectionChanged += new SelectionChangedEventHandler(_selectEmployeeListBox_SelectionChanged);
        }

        void _selectEmployeeListBox_SelectionChanged(object sender, SelectionChangedEventArgs e) {
            int selectedEmployeeId = ((EmployeeForSelection)_selectEmployeeListBox.SelectedItem).EmployeeId;
            _currentEmployee = _mgr.Employees.Where(
                empl => empl.EmployeeID == selectedEmployeeId).FirstOrNullEntity<Employee>();
            this.DataContext = _currentEmployee;
            _managerComboBox.SelectedItem = _currentEmployee.Manager;
        }

        public void LoadSupportingData() {
            _managers.Clear();
            _managers.Insert(0, _mgr.GetNullEntity<Employee>());
            var query = _mgr.Employees.OrderBy(e => e.LastName);
            foreach (Employee anEmployee in query) {
                _managers.Add(anEmployee);
            }
        }

        public void LoadData() {
            var query = from e in _mgr.Employees
                        orderby e.LastName
                        select new { e.EmployeeID, e.LastName, e.FirstName, LastFirst = e.LastName + ", " + e.FirstName };
            foreach (var anEmployeeSelector in query) {
                _employeesForSelection.Add(new EmployeeForSelection(
                    anEmployeeSelector.EmployeeID,
                    anEmployeeSelector.LastName,
                    anEmployeeSelector.FirstName,
                    anEmployeeSelector.LastFirst));
            }
            _selectEmployeeListBox.ItemsSource = _employeesForSelection;

            //_selectEmployeeListBox.DisplayMemberPath = "LastFirst";
        }

        private void ConfigureManagerComboBox() {
            _managerComboBox.ItemsSource = _managers;
            _managerComboBox.DisplayMemberPath = Employee.PathFor(e => e.LastFirst);
        }

        public Employee CurrentEmployee {
            get {
                return _currentEmployee;
            }
        }

        #region Private Fields
        NorthwindIBEntities _mgr = new NorthwindIBEntities();
        Employee _currentEmployee;
        ObservableCollection<EmployeeForSelection> _employeesForSelection =
            new ObservableCollection<EmployeeForSelection>();
        ObservableCollection<Employee> _managers =
            new ObservableCollection<Employee>();
        #endregion Private Fields
    }

    class EmployeeForSelection {
        public EmployeeForSelection(int employeeId, string lastName, string firstName, string lastFirst) {
            EmployeeId = employeeId;
            LastName = lastName;
            FirstName = firstName;
            LastFirst = lastFirst;
      }

        public int EmployeeId {
            get { return _employeeID; }
            set { _employeeID = value; }
        }

        public string LastName {
          get { return _lastName; }
          set { _lastName = value; }
        }

        public string FirstName {
          get { return _firstName; }
          set { _firstName = value; }
        }

        public string LastFirst {
            get { return _lastFirst; }
            set { _lastFirst = value; }
        }

        #region Private Fields
        int _employeeID;
        string _lastName;
        string _firstName;
        string _lastFirst;
        #endregion Private Fields

    }
}