﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using DomainModel;
using IdeaBlade.EntityModel;
using System.Collections.ObjectModel;

namespace LiteHausWPF {
    /// <summary>
    /// Interaction logic for SingleFormAppWindow02.xaml
    /// </summary>
    public partial class SingleFormAppWindow02 : Window {
        public SingleFormAppWindow02() {
            this.InitializeComponent();
            InitializeTabControl();
            ConfigureMenuStrip();
            InitializeTabPageDictionaries();
            InitializePersistence();
            InitializeSavingService();
            //ConfigureMiscellaneousHandlers();
        }

        private void InitializePersistence() {
          DynamicInterceptors.AddDynamicEntityInterceptors();
          _entityManager = new NorthwindIBEntities();
        }

        private void InitializeTabControl() {
            _mainTabControl.Visibility = Visibility.Hidden;
        }

        #region Configure Menu
        private void ConfigureMenuStrip() {
            EstablishMenuHandlers();
            _entitiesEmployeesMenuItem.IsChecked = false;
            _entitiesCustomersMenuItem.IsChecked = false;
            _entitiesOrdersMenuItem.IsChecked = false;
            _entitiesProductsMenuItem.IsChecked = false;
        }

        private void EstablishMenuHandlers() {
            _entitiesEmployeesMenuItem.Click += new RoutedEventHandler(this.entityToolStripMenuItem_Click);
            _entitiesCustomersMenuItem.Click += new RoutedEventHandler(this.entityToolStripMenuItem_Click);
            _entitiesOrdersMenuItem.Click += new RoutedEventHandler(this.entityToolStripMenuItem_Click);
            _entitiesProductsMenuItem.Click += new RoutedEventHandler(this.entityToolStripMenuItem_Click);
            _entitiesUndisplayAllMenuItem.Click += new RoutedEventHandler(this.entityToolStripMenuItem_Click);
        }

        private void entityToolStripMenuItem_Click(object sender, EventArgs e) {
            if (sender.Equals(_entitiesUndisplayAllMenuItem)) {
                UndisplayDisplayedEntities();
            }
            else {
                DisplayEntities((MenuItem)sender);
            }
        }

        private void _fileExitMenuItem_Click(object sender, RoutedEventArgs e) {
            this.Close();
        }

        #region Persistence Menu

        //Click="_persistenceSaveAllChangesMenuItem_Click"
        //Click="_persistenceDiscardAllChangesMenuItem_Click

        private void _persistenceSaveAllChangesMenuItem_Click(object sender, RoutedEventArgs e) {

            // Need WPF equivalent of this.Validate() to commit changes...
            //
            // See MacDonald p.514; but his solution isn't general enough
            //
            // Here's an idea, but I had to specify that I wanted the binding
            // expression for the TextProperty of a TextBox, and really I don't know
            // what sort of control will have the focus.
            //
            //IInputElement scopeOfFocus = FocusManager.GetFocusedElement(
            //    (TabItem)_mainTabControl.SelectedItem);
            //IInputElement itemWithFocus = FocusManager.GetFocusedElement(
            //    (TabItem)scopeOfFocus);
            //((Control)itemWithFocus).GetBindingExpression(TextBox.TextProperty).UpdateSource();

            _savingService.SaveAll();
        }

        private void _persistenceDiscardAllChangesMenuItem_Click(object sender, RoutedEventArgs e) {
            _entityManager.RejectChanges();
        }

        #endregion Persistence Menu


        #endregion Configure Menu

        #region Display Methods

        private void UndisplayDisplayedEntities() {
            foreach (MenuItem aMenuItem in _entitiesMenuItem.Items) {
                if (aMenuItem.IsChecked) {
                    DisplayEntities(aMenuItem);
                }
            }
        }

        private void DisplayEntities(MenuItem pMenuItem) {
            bool currentlyDisplayed = pMenuItem.IsChecked;
            pMenuItem.IsChecked = !pMenuItem.IsChecked;
            TabItemBase aTabItem = _entityDictionary[pMenuItem];
            //aTabItem.BackColor = this.BackColor;
            SetDisplayStateOfTabPage(currentlyDisplayed, aTabItem);
        }

        private void SetDisplayStateOfTabPage(bool currentlyDisplayed, 
              TabItemBase pTabItemBase) {
            if (currentlyDisplayed) {
                RemoveTabPage(pTabItemBase);
            }
            else {
                DisplayTabPage(pTabItemBase);
            }
        }

        private void DisplayTabPage(TabItemBase pTabItem) {
            pTabItem.LoadData();
            ConfigureMainTabControl(pTabItem);
        }

        private void ConfigureMainTabControl(TabItemBase pTabItem) {
            _mainTabControl.Items.Add(pTabItem);
            _mainTabControl.SelectedItem = pTabItem;
            _mainTabControl.Visibility = Visibility.Visible;
        }

        private void RemoveTabPage(TabItemBase pTabItem) {
            _mainTabControl.Items.Remove(pTabItem);
            if (_mainTabControl.Items.Count == 0) {
                _mainTabControl.Visibility = Visibility.Visible;
            }
        }

        #endregion Display Methods

        private void InitializeTabPageDictionaries() {
            _entityDictionary.Add(_entitiesEmployeesMenuItem, new EmployeeTabItem());
            _entityDictionary.Add(_entitiesCustomersMenuItem, new CustomersTabItem());
            _entityDictionary.Add(_entitiesOrdersMenuItem, new OrderTabItem());
            _entityDictionary.Add(_entitiesProductsMenuItem, new ProductsTabItem());
        }

        private void InitializeSavingService() {
            _savingService = new SavingService(_entityManager);
        }

        #region Private Fields
        NorthwindIBEntities _entityManager;
        Dictionary<MenuItem, TabItemBase> _entityDictionary = 
            new Dictionary<MenuItem, TabItemBase>();
        SavingService _savingService;
        #endregion Private Fields

    }
}