﻿#region using
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

using IdeaBlade.EntityModel;
using DomainModel;
using System.Windows;

#endregion

namespace LiteHausWPF {
    /// <summary>
  /// Service class to perform save
  /// </summary>
  /// <remarks>
  /// A primitive example of such a service.
  /// </remarks>
  public class SavingService {

    #region Constructor
    /// <summary>Ctor.</summary>
    /// <param name="entityManager">Provide Saving services for this <see cref="T:EntityManager"/>.</param>
    public SavingService(NorthwindIBEntities entityManager) {
      _entityManager = entityManager;
    }
    #endregion Constructor

    #region SaveAll() method

    /// <summary>Save all entities with pending changes.</summary>
    public void SaveAll() {
      if (!_entityManager.HasChanges()) {
        DisplayNothingToSave();
        return;
      }
      try {
        _entityManager.Saving += SavingHandler;
        SaveResult result = _entityManager.TrySaveChanges();
        DisplaySaveSucceeded();
      }
      catch (EntityManagerSaveException saveException) {
        MessageBox.Show("Exception thrown during save: " + saveException.Message);
      }
      finally {
        _entityManager.Saving -= SavingHandler;
      }
    }

    #endregion SaveAll() method

    #region Display Methods

    private static void DisplayNothingToSave() {
      MessageBox.Show("Nothing to save Succeeded", "Save cancelled");
    }

    private static void DisplaySaveSucceeded() {
      MessageBox.Show("Save Succeeded", "Saved");
    }

    [System.Diagnostics.DebuggerNonUserCode()]
    private static void DisplaySaveFailed(Exception e) {
      Exception anException = e.InnerException;
      if (anException == null) {
        anException = e; // just in case
      }
      string msgFmt =
        "Save failed due to exception of type '{0}':{1}{1}{2}";
      MessageBox.Show(
        String.Format(msgFmt, anException.GetType(), Environment.NewLine, anException.Message),
        "Save failed");
    }
    #endregion

    #region SavingHandler and related methods
    [System.Diagnostics.DebuggerNonUserCode()]
    protected virtual void SavingHandler(object sender, EntitySavingEventArgs e) {
      CheckIfSomethingToSave(e);
      //VerifyEntities(e.Entities);
    }

    [System.Diagnostics.DebuggerNonUserCode()]
    private static void CheckIfSomethingToSave(EntitySavingEventArgs e) {
      if (e.Entities.Count == 0) {
        throw new NothingToSaveException();
      }
    }

    //[System.Diagnostics.DebuggerNonUserCode()]
    //private void VerifyEntities(IList<Entity> pEntities) {
    //  foreach (Entity anEntity in pEntities) {
    //    VerifyAnEntity(anEntity);
    //  }
    //}

    //[System.Diagnostics.DebuggerNonUserCode()]
    //private void VerifyAnEntity(Entity pEntity) {
    //  BaseEntity anEntity = pEntity as BaseEntity;
    //  if (anEntity != null) {
    //    VerifierResultCollection results;
    //    results = anEntity.Verify();
    //    if (!results.AreOk) {
    //      string emsg = anEntity.VerifierResultsAsString;
    //      throw new ValidationException(emsg);
    //    }
    //  }
    //}
    #endregion

    #region Exception classes

    /// <summary>Nothing-to-Save exception.</summary>
    public class NothingToSaveException : Exception {
      public NothingToSaveException()
        : base("Nothing to save.") {
      }
    }

    /// <summary>Entity validation exception.</summary>
    public class ValidationException : Exception {
      public ValidationException(string pMessage)
        : base(pMessage) {
      }
    }
    #endregion

    #region Private Fields
    private NorthwindIBEntities _entityManager;
    #endregion Private Fields

  }
}
