﻿Imports IdeaBlade.EntityModel
Imports IdeaBlade.Core

''' <summary>
''' The developer class representing the Customer business object.
''' </summary>
Partial Public Class Customer

#Region "PropertyInterceptors"

    ' Force the City to upper case for display purposes.
    <AfterGet(EntityPropertyNames.City)>
    Public Function UppercaseCity(ByVal city As String) As String
        If Not String.IsNullOrEmpty(city) Then
            Return city.ToUpper()
        Else
            Return city
        End If
    End Function

    ' Log the set of any customer property.
    <AfterSet>
    Public Sub AfterSetAnyProperty(ByVal args As IEntityPropertyInterceptorArgs)
        Dim msg As String = String.Format("Setting {0} to {1}", args.EntityProperty.Name, If(args.Value Is Nothing, "null", args.Value.ToString()))
        TraceLogger(msg)
    End Sub

    ' Force the ContactName to upper case when set.
    <BeforeSet("ContactName", Key:="C1")>
    Public Sub BeforeSetContactName(ByVal args As DataEntityPropertySetInterceptorArgs(Of Customer, String))
        If args.Value IsNot Nothing Then
            args.Value = args.Value.ToUpper()
        End If
    End Sub

    ' Log when ContactName is set.
    <AfterSet("ContactName", Key:="C2")>
    Public Sub AfterSetContactName(ByVal args As DataEntityPropertySetInterceptorArgs(Of Customer, String))
        Dim msg As String = String.Format("Set {0} to {1}", args.DataEntityProperty.Name, If(args.Value Is Nothing, "null", args.Value))
        TraceLogger(msg)
    End Sub


    Private Shared Sub TraceLogger(ByVal msg As String)
        TraceFns.WriteLine(msg)
    End Sub

#End Region

End Class

