﻿Imports System.Text
Imports Microsoft.Windows.Controls

Imports DomainModel
Imports IdeaBlade.EntityModel
Imports System.Collections.ObjectModel

''' <summary>
''' Interaction logic for EmployeesUserControl.xaml
''' </summary>
Partial Public Class EmployeesUserControl
    Public Sub New()
        Me.InitializeComponent()
        ConfigureEmployeeSelectorListBox()
        ConfigureManagerComboBox()
    End Sub

    Private Sub ConfigureEmployeeSelectorListBox()
        '_selectEmployeeListBox.DisplayMemberPath = Employee.PathFor(e => e.LastFirst);
        AddHandler _selectEmployeeListBox.SelectionChanged, AddressOf _selectEmployeeListBox_SelectionChanged
    End Sub

    Private Sub _selectEmployeeListBox_SelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
        Dim selectedEmployeeId As Integer = (CType(_selectEmployeeListBox.SelectedItem, EmployeeForSelection)).EmployeeId
        _currentEmployee = _mgr.Employees.Where(Function(empl) empl.EmployeeID = selectedEmployeeId).FirstOrNullEntity()
        Me.DataContext = _currentEmployee
        _managerComboBox.SelectedItem = _currentEmployee.Manager
    End Sub

    Public Sub LoadSupportingData()
        _managers.Clear()
        _managers.Insert(0, _mgr.GetNullEntity(Of Employee)())
        Dim query = _mgr.Employees.OrderBy(Function(e) e.LastName)
        For Each anEmployee As Employee In query
            _managers.Add(anEmployee)
        Next anEmployee
    End Sub

    Public Sub LoadData()
        Dim query = From e In _mgr.Employees
                    Order By e.LastName
                    Select New With {Key e.EmployeeID, Key e.LastName, Key e.FirstName, Key .LastFirst = e.LastName & ", " & e.FirstName}
        For Each anEmployeeSelector In query
            _employeesForSelection.Add(New EmployeeForSelection(anEmployeeSelector.EmployeeID, anEmployeeSelector.LastName, anEmployeeSelector.FirstName, anEmployeeSelector.LastFirst))
        Next anEmployeeSelector
        _selectEmployeeListBox.ItemsSource = _employeesForSelection

        '_selectEmployeeListBox.DisplayMemberPath = "LastFirst";
    End Sub

    Private Sub ConfigureManagerComboBox()
        _managerComboBox.ItemsSource = _managers
        _managerComboBox.DisplayMemberPath = Employee.PathFor(Function(e) e.LastFirst)
    End Sub

    Public ReadOnly Property CurrentEmployee() As Employee
        Get
            Return _currentEmployee
        End Get
    End Property

#Region "Private Fields"
    Private _mgr As New NorthwindIBEntities()
    Private _currentEmployee As Employee
    Private _employeesForSelection As New ObservableCollection(Of EmployeeForSelection)()
    Private _managers As New ObservableCollection(Of Employee)()
#End Region ' Private Fields
End Class

Friend Class EmployeeForSelection
    Public Sub New(ByVal employeeId As Integer, ByVal lastName As String, ByVal firstName As String, ByVal lastFirst As String)
        Me.EmployeeId = employeeId
        Me.LastName = lastName
        Me.FirstName = firstName
        Me.LastFirst = lastFirst
    End Sub

    Public Property EmployeeId() As Integer
        Get
            Return _employeeID
        End Get
        Set(ByVal value As Integer)
            _employeeID = value
        End Set
    End Property

    Public Property LastName() As String
        Get
            Return _lastName
        End Get
        Set(ByVal value As String)
            _lastName = value
        End Set
    End Property

    Public Property FirstName() As String
        Get
            Return _firstName
        End Get
        Set(ByVal value As String)
            _firstName = value
        End Set
    End Property

    Public Property LastFirst() As String
        Get
            Return _lastFirst
        End Get
        Set(ByVal value As String)
            _lastFirst = value
        End Set
    End Property

#Region "Private Fields"
    Private _employeeID As Integer
    Private _lastName As String
    Private _firstName As String
    Private _lastFirst As String
#End Region ' Private Fields

End Class