﻿Imports System.Text

Imports DomainModel
Imports IdeaBlade.EntityModel
Imports System.Collections.ObjectModel

''' <summary>
''' Interaction logic for SingleFormAppWindow02.xaml
''' </summary>
Partial Public Class SingleFormAppWindow02
    Inherits Window
    Public Sub New()
        Me.InitializeComponent()
        InitializeTabControl()
        ConfigureMenuStrip()
        InitializeTabPageDictionaries()
        InitializePersistence()
        InitializeSavingService()
        'ConfigureMiscellaneousHandlers();
    End Sub

    Private Sub InitializePersistence()
        DynamicInterceptors.AddDynamicEntityInterceptors()
        _entityManager = New NorthwindIBEntities()
    End Sub

    Private Sub InitializeTabControl()
        _mainTabControl.Visibility = Visibility.Hidden
    End Sub

#Region "Configure Menu"
    Private Sub ConfigureMenuStrip()
        EstablishMenuHandlers()
        _entitiesEmployeesMenuItem.IsChecked = False
        _entitiesCustomersMenuItem.IsChecked = False
        _entitiesOrdersMenuItem.IsChecked = False
        _entitiesProductsMenuItem.IsChecked = False
    End Sub

    Private Sub EstablishMenuHandlers()
        AddHandler _entitiesEmployeesMenuItem.Click, AddressOf entityToolStripMenuItem_Click
        AddHandler _entitiesCustomersMenuItem.Click, AddressOf entityToolStripMenuItem_Click
        AddHandler _entitiesOrdersMenuItem.Click, AddressOf entityToolStripMenuItem_Click
        AddHandler _entitiesProductsMenuItem.Click, AddressOf entityToolStripMenuItem_Click
        AddHandler _entitiesUndisplayAllMenuItem.Click, AddressOf entityToolStripMenuItem_Click
    End Sub

    Private Sub entityToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs)
        If sender.Equals(_entitiesUndisplayAllMenuItem) Then
            UndisplayDisplayedEntities()
        Else
            DisplayEntities(CType(sender, MenuItem))
        End If
    End Sub

    Private Sub _fileExitMenuItem_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Me.Close()
    End Sub

#Region "Persistence Menu"

    'Click="_persistenceSaveAllChangesMenuItem_Click"
    'Click="_persistenceDiscardAllChangesMenuItem_Click

    Private Sub _persistenceSaveAllChangesMenuItem_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)

        ' Need WPF equivalent of this.Validate() to commit changes...
        '
        ' See MacDonald p.514; but his solution isn't general enough
        '
        ' Here's an idea, but I had to specify that I wanted the binding
        ' expression for the TextProperty of a TextBox, and really I don't know
        ' what sort of control will have the focus.
        '
        'IInputElement scopeOfFocus = FocusManager.GetFocusedElement(
        '    (TabItem)_mainTabControl.SelectedItem);
        'IInputElement itemWithFocus = FocusManager.GetFocusedElement(
        '    (TabItem)scopeOfFocus);
        '((Control)itemWithFocus).GetBindingExpression(TextBox.TextProperty).UpdateSource();

        _savingService.SaveAll()
    End Sub

    Private Sub _persistenceDiscardAllChangesMenuItem_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        _entityManager.RejectChanges()
    End Sub

#End Region ' Persistence Menu


#End Region ' Configure Menu

#Region "Display Methods"

    Private Sub UndisplayDisplayedEntities()
        For Each aMenuItem As MenuItem In _entitiesMenuItem.Items
            If aMenuItem.IsChecked Then
                DisplayEntities(aMenuItem)
            End If
        Next aMenuItem
    End Sub

    Private Sub DisplayEntities(ByVal pMenuItem As MenuItem)
        Dim currentlyDisplayed As Boolean = pMenuItem.IsChecked
        pMenuItem.IsChecked = Not pMenuItem.IsChecked
        Dim aTabItem As TabItemBase = _entityDictionary(pMenuItem)
        'aTabItem.BackColor = this.BackColor;
        SetDisplayStateOfTabPage(currentlyDisplayed, aTabItem)
    End Sub

    Private Sub SetDisplayStateOfTabPage(ByVal currentlyDisplayed As Boolean, ByVal pTabItemBase As TabItemBase)
        If currentlyDisplayed Then
            RemoveTabPage(pTabItemBase)
        Else
            DisplayTabPage(pTabItemBase)
        End If
    End Sub

    Private Sub DisplayTabPage(ByVal pTabItem As TabItemBase)
        pTabItem.LoadData()
        ConfigureMainTabControl(pTabItem)
    End Sub

    Private Sub ConfigureMainTabControl(ByVal pTabItem As TabItemBase)
        _mainTabControl.Items.Add(pTabItem)
        _mainTabControl.SelectedItem = pTabItem
        _mainTabControl.Visibility = Visibility.Visible
    End Sub

    Private Sub RemoveTabPage(ByVal pTabItem As TabItemBase)
        _mainTabControl.Items.Remove(pTabItem)
        If _mainTabControl.Items.Count = 0 Then
            _mainTabControl.Visibility = Visibility.Visible
        End If
    End Sub

#End Region ' Display Methods

    Private Sub InitializeTabPageDictionaries()
        _entityDictionary.Add(_entitiesEmployeesMenuItem, New EmployeeTabItem())
        _entityDictionary.Add(_entitiesCustomersMenuItem, New CustomersTabItem())
        _entityDictionary.Add(_entitiesOrdersMenuItem, New OrderTabItem())
        _entityDictionary.Add(_entitiesProductsMenuItem, New ProductsTabItem())
    End Sub

    Private Sub InitializeSavingService()
        _savingService = New SavingService(_entityManager)
    End Sub

#Region "Private Fields"
    Private _entityManager As NorthwindIBEntities
    Private _entityDictionary As New Dictionary(Of MenuItem, TabItemBase)()
    Private _savingService As SavingService
#End Region ' Private Fields

End Class