﻿#Region "using"
Imports System.ComponentModel
Imports System.Text

Imports IdeaBlade.EntityModel
Imports DomainModel

#End Region

''' <summary>
''' Service class to perform save
''' </summary>
''' <remarks>
''' A primitive example of such a service.
''' </remarks>
Public Class SavingService

#Region "Constructor"
    ''' <summary>Ctor.</summary>
    ''' <param name="entityManager">Provide Saving services for this <see cref="T:EntityManager"/>.</param>
    Public Sub New(ByVal entityManager As NorthwindIBEntities)
        _entityManager = entityManager
    End Sub
#End Region ' Constructor

#Region "SaveAll() method"

    ''' <summary>Save all entities with pending changes.</summary>
    Public Sub SaveAll()
        If Not _entityManager.HasChanges() Then
            DisplayNothingToSave()
            Return
        End If
        Try
            AddHandler _entityManager.Saving, AddressOf SavingHandler
            _entityManager.SaveChanges()
            DisplaySaveSucceeded()
        Catch saveException As EntityManagerSaveException
            MessageBox.Show("Exception thrown during save: " & saveException.Message)
        Finally
            RemoveHandler _entityManager.Saving, AddressOf SavingHandler
        End Try
    End Sub

#End Region ' SaveAll() method

#Region "Display Methods"

    Private Shared Sub DisplayNothingToSave()
        MessageBox.Show("Nothing to save Succeeded", "Save cancelled")
    End Sub

    Private Shared Sub DisplaySaveSucceeded()
        MessageBox.Show("Save Succeeded", "Saved")
    End Sub

    <DebuggerNonUserCode()>
    Private Shared Sub DisplaySaveFailed(ByVal e As Exception)
        Dim anException As Exception = e.InnerException
        If anException Is Nothing Then
            anException = e ' just in case
        End If
        Dim msgFmt As String = "Save failed due to exception of type '{0}':{1}{1}{2}"
        MessageBox.Show(String.Format(msgFmt, anException.GetType(), Environment.NewLine, anException.Message), "Save failed")
    End Sub
#End Region

#Region "SavingHandler and related methods"
    <DebuggerNonUserCode()>
    Protected Overridable Sub SavingHandler(ByVal sender As Object, ByVal e As EntitySavingEventArgs)
        CheckIfSomethingToSave(e)
        'VerifyEntities(e.Entities);
    End Sub

    <DebuggerNonUserCode()>
    Private Shared Sub CheckIfSomethingToSave(ByVal e As EntitySavingEventArgs)
        If e.Entities.Count = 0 Then
            Throw New NothingToSaveException()
        End If
    End Sub

    '[System.Diagnostics.DebuggerNonUserCode()]
    'private void VerifyEntities(IList<Entity> pEntities) {
    '  foreach (Entity anEntity in pEntities) {
    '    VerifyAnEntity(anEntity);
    '  }
    '}

    '[System.Diagnostics.DebuggerNonUserCode()]
    'private void VerifyAnEntity(Entity pEntity) {
    '  BaseEntity anEntity = pEntity as BaseEntity;
    '  if (anEntity != null) {
    '    VerifierResultCollection results;
    '    results = anEntity.Verify();
    '    if (!results.AreOk) {
    '      string emsg = anEntity.VerifierResultsAsString;
    '      throw new ValidationException(emsg);
    '    }
    '  }
    '}
#End Region

#Region "Exception classes"

    ''' <summary>Nothing-to-Save exception.</summary>
    Public Class NothingToSaveException
        Inherits Exception
        Public Sub New()
            MyBase.New("Nothing to save.")
        End Sub
    End Class

    ''' <summary>Entity validation exception.</summary>
    Public Class ValidationException
        Inherits Exception
        Public Sub New(ByVal pMessage As String)
            MyBase.New(pMessage)
        End Sub
    End Class
#End Region

#Region "Private Fields"
    Private _entityManager As NorthwindIBEntities
#End Region ' Private Fields

End Class
